@@ -24,11 +24,6 @@ do
 	echo "done"
 done
 
-echo -n "Removing unused music files ... "
-rm -f "$DISTDIR"/data/music/airship_remix.*
-rm -f "$DISTDIR"/data/music/battle_theme.*
-echo "done"
-
 echo -n "Creating $DISTDIR.tar.gz ... "
 tar czf $DISTDIR.tar.gz $DISTDIR
 echo "done"
