@@ -237,52 +237,53 @@ Menu::update()
     effect_progress = 0.0f;
   }
 
+  Controller *controller = g_jk_controller->get_main_controller();
   /** check main input controller... */
-  if(g_main_controller->pressed(Controller::UP)) {
+  if(controller->pressed(Controller::UP)) {
     menuaction = MENU_ACTION_UP;
     menu_repeat_time = real_time + MENU_REPEAT_INITIAL;
   }
-  if(g_main_controller->hold(Controller::UP) &&
+  if(controller->hold(Controller::UP) &&
      menu_repeat_time != 0 && real_time > menu_repeat_time) {
     menuaction = MENU_ACTION_UP;
     menu_repeat_time = real_time + MENU_REPEAT_RATE;
   }
 
-  if(g_main_controller->pressed(Controller::DOWN)) {
+  if(controller->pressed(Controller::DOWN)) {
     menuaction = MENU_ACTION_DOWN;
     menu_repeat_time = real_time + MENU_REPEAT_INITIAL;
   }
-  if(g_main_controller->hold(Controller::DOWN) &&
+  if(controller->hold(Controller::DOWN) &&
      menu_repeat_time != 0 && real_time > menu_repeat_time) {
     menuaction = MENU_ACTION_DOWN;
     menu_repeat_time = real_time + MENU_REPEAT_RATE;
   }
 
-  if(g_main_controller->pressed(Controller::LEFT)) {
+  if(controller->pressed(Controller::LEFT)) {
     menuaction = MENU_ACTION_LEFT;
     menu_repeat_time = real_time + MENU_REPEAT_INITIAL;
   }
-  if(g_main_controller->hold(Controller::LEFT) &&
+  if(controller->hold(Controller::LEFT) &&
      menu_repeat_time != 0 && real_time > menu_repeat_time) {
     menuaction = MENU_ACTION_LEFT;
     menu_repeat_time = real_time + MENU_REPEAT_RATE;
   }
 
-  if(g_main_controller->pressed(Controller::RIGHT)) {
+  if(controller->pressed(Controller::RIGHT)) {
     menuaction = MENU_ACTION_RIGHT;
     menu_repeat_time = real_time + MENU_REPEAT_INITIAL;
   }
-  if(g_main_controller->hold(Controller::RIGHT) &&
+  if(controller->hold(Controller::RIGHT) &&
      menu_repeat_time != 0 && real_time > menu_repeat_time) {
     menuaction = MENU_ACTION_RIGHT;
     menu_repeat_time = real_time + MENU_REPEAT_RATE;
   }
 
-  if(g_main_controller->pressed(Controller::ACTION)
-     || g_main_controller->pressed(Controller::MENU_SELECT)) {
+  if(controller->pressed(Controller::ACTION)
+     || controller->pressed(Controller::MENU_SELECT)) {
     menuaction = MENU_ACTION_HIT;
   }
-  if(g_main_controller->pressed(Controller::PAUSE_MENU)) {
+  if(controller->pressed(Controller::PAUSE_MENU)) {
     menuaction = MENU_ACTION_BACK;
   }
 
@@ -452,7 +453,6 @@ Menu::draw_item(DrawingContext& context, int index)
   Color text_color = default_color;
   float x_pos       = pos.x;
   float y_pos       = pos.y + 24*index - menu_height/2 + 12;
-  int shadow_size = 2;
   int text_width  = int(Resources::normal_font->get_text_width(pitem.text));
   int input_width = int(Resources::normal_font->get_text_width(pitem.input) + 10);
   int list_width = 0;
@@ -469,7 +469,6 @@ Menu::draw_item(DrawingContext& context, int index)
 
   if(index == active_item)
   {
-    shadow_size = 3;
     text_color = active_color;
   }
 
@@ -623,6 +622,9 @@ Menu::get_width() const
       Resources::big_font->get_text_width(items[i]->input) + 16;
     if(items[i]->kind == MN_TOGGLE)
       w += 32;
+    if (items[i]->kind == MN_STRINGSELECT)
+      w += font->get_text_width(items[i]->list[items[i]->selected]) + 32;
+    
 
     if(w > menu_width)
       menu_width = w;
@@ -777,6 +779,7 @@ Menu::event(const SDL_Event& event)
 
   switch(event.type) {
     case SDL_MOUSEBUTTONDOWN:
+    if(event.button.button == SDL_BUTTON_LEFT)
     {
       int x = int(event.motion.x * float(SCREEN_WIDTH)/g_screen->w);
       int y = int(event.motion.y * float(SCREEN_HEIGHT)/g_screen->h);
