@@ -1,12 +1,10 @@
-//  $Id: gradient.hpp 4063 2006-07-21 21:05:23Z anmaster $
-//
 //  SuperTux
 //  Copyright (C) 2006 Matthias Braun <matze@braunis.de>
 //
-//  This program is free software; you can redistribute it and/or
-//  modify it under the terms of the GNU General Public License
-//  as published by the Free Software Foundation; either version 2
-//  of the License, or (at your option) any later version.
+//  This program is free software: you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation, either version 3 of the License, or
+//  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,33 +12,24 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#ifndef SUPERTUX_GRADIENT_H
-#define SUPERTUX_GRADIENT_H
+#ifndef HEADER_SUPERTUX_OBJECT_GRADIENT_HPP
+#define HEADER_SUPERTUX_OBJECT_GRADIENT_HPP
 
-#include <memory>
-#include "video/surface.hpp"
+#include "supertux/game_object.hpp"
+#include "util/reader_fwd.hpp"
 #include "video/drawing_context.hpp"
-#include "game_object.hpp"
-#include "serializable.hpp"
 
 class DisplayManager;
 
-namespace lisp {
-class Lisp;
-}
-
-class Gradient : public GameObject, public Serializable
+class Gradient : public GameObject
 {
 public:
   Gradient();
-  Gradient(const lisp::Lisp& reader);
+  Gradient(const Reader& reader);
   virtual ~Gradient();
 
-  virtual void write(lisp::Writer& writer);
-
   void set_gradient(Color top, Color bottom);
 
   Color get_gradient_top() const
@@ -55,7 +44,10 @@ public:
 
 private:
   int layer;
-  Color gradient_top, gradient_bottom;
+  Color gradient_top;
+  Color gradient_bottom;
 };
 
 #endif /*SUPERTUX_BACKGROUND_H*/
+
+/* EOF */
