@@ -1,12 +1,10 @@
-//  $Id: powerup.hpp 4063 2006-07-21 21:05:23Z anmaster $
-//
 //  SuperTux
 //  Copyright (C) 2006 Matthias Braun <matze@braunis.de>
 //
-//  This program is free software; you can redistribute it and/or
-//  modify it under the terms of the GNU General Public License
-//  as published by the Free Software Foundation; either version 2
-//  of the License, or (at your option) any later version.
+//  This program is free software: you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation, either version 3 of the License, or
+//  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,21 +12,17 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#ifndef __POWERUP_H__
-#define __POWERUP_H__
+#ifndef HEADER_SUPERTUX_OBJECT_POWERUP_HPP
+#define HEADER_SUPERTUX_OBJECT_POWERUP_HPP
 
 #include "object/moving_sprite.hpp"
-#include "lisp/lisp.hpp"
-#include "collision_hit.hpp"
-#include "physic.hpp"
 
 class PowerUp : public MovingSprite
 {
 public:
-  PowerUp(const lisp::Lisp& lisp);
+  PowerUp(const Reader& lisp);
 
   virtual void update(float elapsed_time);
   virtual void collision_solid(const CollisionHit& hit);
@@ -41,3 +35,5 @@ private:
 };
 
 #endif
+
+/* EOF */
