@@ -1,12 +1,10 @@
-//  $Id: specialriser.cpp 4059 2006-07-21 11:45:14Z matzebraun $
-//
 //  SuperTux
 //  Copyright (C) 2006 Matthias Braun <matze@braunis.de>
 //
-//  This program is free software; you can redistribute it and/or
-//  modify it under the terms of the GNU General Public License
-//  as published by the Free Software Foundation; either version 2
-//  of the License, or (at your option) any later version.
+//  This program is free software: you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation, either version 3 of the License, or
+//  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,20 +12,15 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include "video/drawing_context.hpp"
+#include "object/specialriser.hpp"
+#include "supertux/sector.hpp"
 
-#include <math.h>
-#include "specialriser.hpp"
-#include "resources.hpp"
-#include "camera.hpp"
-#include "sector.hpp"
-#include "sprite/sprite_manager.hpp"
-
-SpecialRiser::SpecialRiser(Vector pos, MovingObject* _child)
-  : child(_child)
+SpecialRiser::SpecialRiser(Vector pos, MovingObject* _child) :
+  offset(),
+  child(_child)
 {
   _child->set_pos(pos - Vector(0, 32));
   offset = 0;
@@ -52,7 +45,9 @@ SpecialRiser::draw(DrawingContext& context)
 {
   context.push_transform();
   context.set_translation(
-      context.get_translation() + Vector(0, -32 + offset));
+    context.get_translation() + Vector(0, -32 + offset));
   child->draw(context);
   context.pop_transform();
 }
+
+/* EOF */
