@@ -34,7 +34,8 @@
 GLRenderer::GLRenderer() :
   desktop_size(-1, -1),
   screen_size(-1, -1),
-  fullscreen_active(false)
+  fullscreen_active(false),
+  last_texture(static_cast<GLuint> (-1))
 {
   Renderer::instance_ = this;
 
@@ -113,10 +114,14 @@ void
 GLRenderer::draw_surface(const DrawingRequest& request)
 {
   const Surface* surface = (const Surface*) request.request_data;
-  boost::shared_ptr<GLTexture> gltexture = boost::dynamic_pointer_cast<GLTexture>(surface->get_texture());
-  GLSurfaceData *surface_data = reinterpret_cast<GLSurfaceData *>(surface->get_surface_data());
+  GLTexture* gltexture = static_cast<GLTexture*>(surface->get_texture().get());
+  GLSurfaceData *surface_data = static_cast<GLSurfaceData*>(surface->get_surface_data());
 
-  glBindTexture(GL_TEXTURE_2D, gltexture->get_handle());
+  GLuint th = gltexture->get_handle();
+  if (th != last_texture) {
+    last_texture = th;
+    glBindTexture(GL_TEXTURE_2D, th);
+  }
   intern_draw(request.pos.x, request.pos.y,
               request.pos.x + surface->get_width(),
               request.pos.y + surface->get_height(),
@@ -148,7 +153,11 @@ GLRenderer::draw_surface_part(const DrawingRequest& request)
   float uv_right = surface_data->get_uv_left() + (uv_width * (surfacepartrequest->source.x + surfacepartrequest->size.x)) / surface->get_width();
   float uv_bottom = surface_data->get_uv_top() + (uv_height * (surfacepartrequest->source.y + surfacepartrequest->size.y)) / surface->get_height();
 
-  glBindTexture(GL_TEXTURE_2D, gltexture->get_handle());
+  GLuint th = gltexture->get_handle();
+  if (th != last_texture) {
+    last_texture = th;
+    glBindTexture(GL_TEXTURE_2D, th);
+  }
   intern_draw(request.pos.x, request.pos.y,
               request.pos.x + surfacepartrequest->size.x,
               request.pos.y + surfacepartrequest->size.y,
@@ -177,9 +186,9 @@ GLRenderer::draw_gradient(const DrawingRequest& request)
 
   float vertices[] = {
     0, 0,
-    SCREEN_WIDTH, 0,
-    SCREEN_WIDTH, SCREEN_HEIGHT,
-    0, SCREEN_HEIGHT
+    float(SCREEN_WIDTH), 0,
+    float(SCREEN_WIDTH), float(SCREEN_HEIGHT),
+    0, float(SCREEN_HEIGHT)
   };
   glVertexPointer(2, GL_FLOAT, 0, vertices);
 
@@ -501,11 +510,12 @@ GLRenderer::apply_config()
   }
 
   Size max_size(1280, 800);
+  Size min_size(640, 480);
 
   if (g_config->magnification == 0.0f) // Magic value that means 'minfill'
   {
     // This scales SCREEN_WIDTH/SCREEN_HEIGHT so that they never excede
-    // max_size.width/max_size.height
+    // max_size.width/max_size.height resp. min_size.width/min_size.height
     if (SCREEN_WIDTH > max_size.width || SCREEN_HEIGHT > max_size.height)
     {
       float scale1  = float(max_size.width)/SCREEN_WIDTH;
@@ -513,7 +523,16 @@ GLRenderer::apply_config()
       float scale   = (scale1 < scale2) ? scale1 : scale2;
       SCREEN_WIDTH  = static_cast<int>(SCREEN_WIDTH  * scale);
       SCREEN_HEIGHT = static_cast<int>(SCREEN_HEIGHT * scale);
+    } 
+    else if (SCREEN_WIDTH < min_size.width || SCREEN_HEIGHT < min_size.height)
+    {
+      float scale1  = float(min_size.width)/SCREEN_WIDTH;
+      float scale2  = float(min_size.height)/SCREEN_HEIGHT;
+      float scale   = (scale1 < scale2) ? scale1 : scale2;
+      SCREEN_WIDTH  = static_cast<int>(SCREEN_WIDTH  * scale);
+      SCREEN_HEIGHT = static_cast<int>(SCREEN_HEIGHT * scale);
     }
+   
 
     glViewport(0, 0, screen_size.width, screen_size.height);
   }
